@@ -904,14 +904,14 @@ namespace ICN
         ZOMBIE2,
         ZOMBIE,
 
-        LAST_VALID_FILE_ICN, // Real ICNs need a special reference to ICN files. Put generated by application ICNs at the end of this enumeration.
+        // Real ICNs need a special reference to ICN files. Put ICNs generated by the engine at the end of this enumeration.
+        LAST_VALID_FILE_ICN,
 
         // system
         ROUTERED,
         YELLOW_FONT,
         YELLOW_SMALLFONT,
         BUTTON_WELL_MAX,
-        BUTTON_BATTLE_ONLY,
         BUTTON_GIFT_GOOD,
         BUTTON_GIFT_EVIL,
         CSLMARKER,
@@ -961,6 +961,8 @@ namespace ICN
         BARBARIAN_CASTLE_CAPTAIN_QUARTERS_LEFT_SIDE,
         SORCERESS_CASTLE_CAPTAIN_QUARTERS_LEFT_SIDE,
 
+        WIZARD_CASTLE_BAY,
+
         GOOD_ARMY_BUTTON,
         GOOD_MARKET_BUTTON,
         EVIL_ARMY_BUTTON,
@@ -1007,25 +1009,15 @@ namespace ICN
         EMPTY_INTERFACE_BUTTON_GOOD,
         EMPTY_INTERFACE_BUTTON_EVIL,
 
-        BUTTON_STANDARD_GAME,
-        BUTTON_CAMPAIGN_GAME,
-        BUTTON_MULTIPLAYER_GAME,
-        BUTTON_LARGE_CANCEL,
-        BUTTON_LARGE_CONFIG,
-        BUTTON_ORIGINAL_CAMPAIGN,
-        BUTTON_EXPANSION_CAMPAIGN,
-        BUTTON_HOT_SEAT,
-        BUTTON_2_PLAYERS,
-        BUTTON_3_PLAYERS,
-        BUTTON_4_PLAYERS,
-        BUTTON_5_PLAYERS,
-        BUTTON_6_PLAYERS,
-
-        BUTTON_NEW_MAP,
-        BUTTON_LOAD_MAP,
+        BUTTONS_NEW_GAME_MENU_GOOD,
+        BUTTONS_NEW_GAME_MENU_EVIL,
+        BUTTONS_EDITOR_MENU_GOOD,
+        BUTTONS_EDITOR_MENU_EVIL,
 
         BUTTONS_FILE_DIALOG_GOOD,
         BUTTONS_FILE_DIALOG_EVIL,
+        BUTTONS_EDITOR_FILE_DIALOG_GOOD,
+        BUTTONS_EDITOR_FILE_DIALOG_EVIL,
         BUTTON_INFO_GOOD,
         BUTTON_INFO_EVIL,
         BUTTON_QUIT_GOOD,
@@ -1051,6 +1043,7 @@ namespace ICN
         BUTTON_SMALL_EXIT_EVIL,
         BUTTON_EXIT_HEROES_MEETING,
         BUTTON_EXIT_TOWN,
+        BUTTON_OKAY_TOWN,
         BUTTON_EXIT_PUZZLE_DIM_DOOR_GOOD,
         BUTTON_EXIT_PUZZLE_DIM_DOOR_EVIL,
         BUTTON_SMALL_DISMISS_GOOD,
@@ -1065,8 +1058,12 @@ namespace ICN
         BUTTON_SMALL_MAX_EVIL,
 
         BUTTON_RESET_GOOD,
+        BUTTON_RESET_EVIL,
         BUTTON_START_GOOD,
 
+        BUTTON_VIRTUAL_KEYBOARD_GOOD,
+        BUTTON_VIRTUAL_KEYBOARD_EVIL,
+
         BUTTON_KINGDOM_EXIT,
         BUTTON_KINGDOM_HEROES,
         BUTTON_KINGDOM_TOWNS,
@@ -1102,15 +1099,10 @@ namespace ICN
         METALLIC_BORDERED_TEXTBOX_GOOD,
         METALLIC_BORDERED_TEXTBOX_EVIL,
 
-        BUTTON_NEW_MAP_EVIL,
-        BUTTON_NEW_MAP_GOOD,
-        BUTTON_SAVE_MAP_EVIL,
-        BUTTON_SAVE_MAP_GOOD,
-        BUTTON_LOAD_MAP_EVIL,
-        BUTTON_LOAD_MAP_GOOD,
-
         SCENIBKG_EVIL,
 
+        EVIL_DIALOG_PLAIN_CORNERS,
+
         // IMPORTANT! Put any new entry just above this one.
         LASTICN
     };
