@@ -107,7 +107,7 @@ namespace
         return 0.0;
     }
 
-    std::pair<int32_t, int> optimalAttackVector( const Battle::Unit & attacker, const Battle::Unit & target, const Battle::Position & attackPos )
+    std::pair<int32_t, Battle::CellDirection> optimalAttackVector( const Battle::Unit & attacker, const Battle::Unit & target, const Battle::Position & attackPos )
     {
         assert( attackPos.isValidForUnit( attacker ) );
         assert( Battle::Board::CanAttackTargetFromPosition( attacker, target, attackPos.GetHead()->GetIndex() ) );
@@ -117,7 +117,7 @@ namespace
         const std::array<const Battle::Cell *, 2> attackCells = { attackPos.GetHead(), attackPos.GetTail() };
         const std::array<const Battle::Cell *, 2> targetCells = { targetPos.GetHead(), targetPos.GetTail() };
 
-        std::pair<int32_t, int> bestAttackVector{ -1, Battle::UNKNOWN };
+        std::pair<int32_t, Battle::CellDirection> bestAttackVector{ -1, Battle::CellDirection::UNKNOWN };
         double bestAttackValue = 0.0;
 
         for ( const Battle::Cell * attackCell : attackCells ) {
@@ -908,7 +908,7 @@ Battle::Actions AI::BattlePlanner::planUnitTurn( Battle::Arena & arena, const Ba
                 const auto [attackTargetIdx, attackDirection] = optimalAttackVector( currentUnit, *target.unit, attackPos );
 
                 actions.emplace_back( Battle::Command::ATTACK, currentUnit.GetUID(), target.unit->GetUID(),
-                                      ( currentUnit.GetHeadIndex() == moveTargetIdx ? -1 : moveTargetIdx ), attackTargetIdx, attackDirection );
+                                      ( currentUnit.GetHeadIndex() == moveTargetIdx ? -1 : moveTargetIdx ), attackTargetIdx, static_cast<int>( attackDirection ) );
 
                 DEBUG_LOG( DBG_BATTLE, DBG_INFO,
                            currentUnit.GetName() << " attacking enemy " << target.unit->GetName() << " from cell " << moveTargetIdx << ", attack vector: "
@@ -1725,7 +1725,7 @@ AI::BattleTargetPair AI::BattlePlanner::meleeUnitDefense( Battle::Arena & arena,
                     Battle::Indexes result;
                     result.reserve( 8 );
 
-                    const std::array<int, 6> priorityDirections = [&currentUnit, frnd]() -> std::array<int, 6> {
+                    const std::array<Battle::CellDirection, 6> priorityDirections = [&currentUnit, frnd]() -> std::array<Battle::CellDirection, 6> {
                         const bool preferToCoverFromTheSide = [&currentUnit, frnd]() {
                             // If the covering unit is not a wide unit, then using this unit to cover the shooter from the side does not give any advantage
                             if ( !currentUnit.isWide() ) {
@@ -1750,17 +1750,21 @@ AI::BattleTargetPair AI::BattlePlanner::meleeUnitDefense( Battle::Arena & arena,
 
                         if ( preferToCoverFromTheSide ) {
                             if ( frnd->isReflect() ) {
-                                return { Battle::TOP_LEFT, Battle::BOTTOM_LEFT, Battle::LEFT, Battle::TOP_RIGHT, Battle::BOTTOM_RIGHT, Battle::RIGHT };
+                                return { Battle::CellDirection::TOP_LEFT,  Battle::CellDirection::BOTTOM_LEFT,  Battle::CellDirection::LEFT,
+                                         Battle::CellDirection::TOP_RIGHT, Battle::CellDirection::BOTTOM_RIGHT, Battle::CellDirection::RIGHT };
                             }
 
-                            return { Battle::TOP_RIGHT, Battle::BOTTOM_RIGHT, Battle::RIGHT, Battle::TOP_LEFT, Battle::BOTTOM_LEFT, Battle::LEFT };
+                            return { Battle::CellDirection::TOP_RIGHT, Battle::CellDirection::BOTTOM_RIGHT, Battle::CellDirection::RIGHT,
+                                     Battle::CellDirection::TOP_LEFT,  Battle::CellDirection::BOTTOM_LEFT,  Battle::CellDirection::LEFT };
                         }
 
                         if ( frnd->isReflect() ) {
-                            return { Battle::LEFT, Battle::TOP_LEFT, Battle::BOTTOM_LEFT, Battle::TOP_RIGHT, Battle::BOTTOM_RIGHT, Battle::RIGHT };
+                            return { Battle::CellDirection::LEFT,      Battle::CellDirection::TOP_LEFT,     Battle::CellDirection::BOTTOM_LEFT,
+                                     Battle::CellDirection::TOP_RIGHT, Battle::CellDirection::BOTTOM_RIGHT, Battle::CellDirection::RIGHT };
                         }
 
-                        return { Battle::RIGHT, Battle::TOP_RIGHT, Battle::BOTTOM_RIGHT, Battle::TOP_LEFT, Battle::BOTTOM_LEFT, Battle::LEFT };
+                        return { Battle::CellDirection::RIGHT,    Battle::CellDirection::TOP_RIGHT,   Battle::CellDirection::BOTTOM_RIGHT,
+                                 Battle::CellDirection::TOP_LEFT, Battle::CellDirection::BOTTOM_LEFT, Battle::CellDirection::LEFT };
                     }();
 
                     for ( const int32_t idx : std::array<int32_t, 2>{ frnd->GetHeadIndex(), frnd->GetTailIndex() } ) {
@@ -1768,7 +1772,7 @@ AI::BattleTargetPair AI::BattlePlanner::meleeUnitDefense( Battle::Arena & arena,
                             continue;
                         }
 
-                        for ( const int dir : priorityDirections ) {
+                        for ( const Battle::CellDirection dir : priorityDirections ) {
                             if ( !Battle::Board::isValidDirection( idx, dir ) ) {
                                 continue;
                             }
@@ -1786,7 +1790,7 @@ AI::BattleTargetPair AI::BattlePlanner::meleeUnitDefense( Battle::Arena & arena,
 
                                 if ( currentUnit.isWide() ) {
                                     // If the shooter is covered in front by a wide unit, then this unit should be located one more cell further away.
-                                    if ( dir == ( frnd->isReflect() ? Battle::LEFT : Battle::RIGHT ) ) {
+                                    if ( dir == ( frnd->isReflect() ? Battle::CellDirection::LEFT : Battle::CellDirection::RIGHT ) ) {
                                         if ( !Battle::Board::isValidDirection( nearbyIdx, dir ) ) {
                                             continue;
                                         }
@@ -1795,7 +1799,7 @@ AI::BattleTargetPair AI::BattlePlanner::meleeUnitDefense( Battle::Arena & arena,
                                     }
                                     // If a wide unit covers a shooter from behind (which is rare, but it can happen) it is necessary to check that the covering unit
                                     // will not be located close to the shooter - which can happen if there is any obstacle in place of the intended tail of the unit.
-                                    else if ( dir == ( frnd->isReflect() ? Battle::RIGHT : Battle::LEFT ) ) {
+                                    else if ( dir == ( frnd->isReflect() ? Battle::CellDirection::RIGHT : Battle::CellDirection::LEFT ) ) {
                                         const Battle::Position pos = Battle::Position::GetPosition( currentUnit, nearbyIdx );
                                         if ( pos.GetHead() == nullptr ) {
                                             continue;
