@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2024                                             *
+ *   Copyright (C) 2019 - 2025                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -29,6 +29,7 @@
 #include <string>
 #include <vector>
 
+#include "color.h"
 #include "monster.h"
 #include "players.h"
 
@@ -57,10 +58,10 @@ public:
 
     Troops & operator=( const Troops & ) = delete;
 
-    void Assign( const Troop * itbeg, const Troop * itend );
-    void Assign( const Troops & );
-    void Insert( const Troops & );
-    void PushBack( const Monster &, uint32_t );
+    void Assign( const Troop * troopsBegin, const Troop * troopsEnd );
+    void Assign( const Troops & troops );
+    void Insert( const Troops & troops );
+    void PushBack( const Monster & mons, const uint32_t count );
     void PopBack();
 
     size_t Size() const
@@ -97,7 +98,7 @@ public:
     void Clean();
     void UpgradeTroops( const Castle & castle ) const;
 
-    Troop * GetFirstValid();
+    Troop * GetFirstValid() const;
     Troop * GetWeakestTroop() const;
     Troop * GetSlowestTroop() const;
 
@@ -172,7 +173,7 @@ public:
 
     static void SwapTroops( Troop &, Troop & );
 
-    static NeutralMonsterJoiningCondition GetJoinSolution( const Heroes &, const Maps::Tile &, const Troop & );
+    static NeutralMonsterJoiningCondition GetJoinSolution( const Heroes & hero, const Maps::Tile & tile, const Troop & troop );
 
     static void drawSingleDetailedMonsterLine( const Troops & troops, int32_t cx, int32_t cy, int32_t width );
     static void drawMultipleMonsterLines( const Troops & troops, int32_t posX, int32_t posY, int32_t lineWidth, bool isCompact, const bool isDetailedView,
@@ -187,7 +188,10 @@ public:
 
     Army & operator=( const Army & ) = delete;
 
-    const Troops & getTroops() const;
+    const Troops & getTroops() const
+    {
+        return *this;
+    }
 
     // Resets the army. If the army doesn't have a commanding hero, then it makes the army empty. Otherwise, if 'defaultArmy' is set to true, then it creates a default
     // army of the commanding hero's faction (several units of level 1 and 2). Otherwise, a minimum army is created, consisting of exactly one monster of the first level
@@ -195,7 +199,7 @@ public:
     void Reset( const bool defaultArmy = false );
     void setFromTile( const Maps::Tile & tile );
 
-    int GetColor() const;
+    PlayerColor GetColor() const;
     int GetControl() const override;
     uint32_t getTotalCount() const;
 
@@ -203,9 +207,9 @@ public:
     bool isStrongerThan( const Army & target, const double safetyRatio = 1.0 ) const;
     bool isMeleeDominantArmy() const;
 
-    void SetColor( int cl )
+    void SetColor( const PlayerColor color )
     {
-        color = cl;
+        _color = color;
     }
 
     int GetMorale() const;
@@ -228,8 +232,12 @@ public:
 
     void JoinStrongestFromArmy( Army & giver );
 
-    // Implements the necessary logic to move unit stacks from army to army in the hero's meeting dialog and in the castle dialog
+    // Implements the necessary logic to move unit stacks from army to army in the heroes meeting dialog and in the castle dialog
     void MoveTroops( Army & from, const int monsterIdToKeep );
+    // Implements the necessary logic to swap all unit stacks from an army to another army in the heroes meeting dialog and in the
+    // castle dialog - provided that there is at least one occupied slot in the castle garrison. It's the caller's responsibility
+    // to ensure that this is indeed the case.
+    void SwapTroops( Army & from );
 
     void SetSpreadFormation( const bool spread )
     {
@@ -265,6 +273,6 @@ private:
     void ArrangeForBattle( const Monster & monster, const uint32_t monstersCount, const int32_t tileIndex, const bool allowUpgrade );
 
     HeroBase * commander;
-    bool _isSpreadCombatFormation;
-    int color;
+    bool _isSpreadCombatFormation{ true };
+    PlayerColor _color{ PlayerColor::NONE };
 };
