@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2024                                             *
+ *   Copyright (C) 2019 - 2025                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2012 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -21,10 +21,8 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
-#ifndef H2BATTLE_ARMY_H
-#define H2BATTLE_ARMY_H
+#pragma once
 
-#include <algorithm> // IWYU pragma: keep
 #include <cassert>
 #include <cstdint>
 #include <iterator>
@@ -39,6 +37,8 @@
 
 class HeroBase;
 
+enum class PlayerColor : uint8_t;
+
 namespace Battle
 {
     class TroopsUidGenerator;
@@ -95,8 +95,8 @@ namespace Battle
                     };
                 }
                 else {
-                    // The build will fail because this lambda does not meet the requirements of UnaryPredicate
-                    return []() {};
+                    // The build should fail because this lambda does not meet the requirements of UnaryPredicate
+                    return []() { assert( 0 ); };
                 }
             };
 
@@ -109,8 +109,8 @@ namespace Battle
 
         Units & operator=( const Units & ) = delete;
 
-        Unit * FindMode( uint32_t mod ) const;
-        Unit * FindUID( uint32_t pid ) const;
+        Unit * FindMode( const uint32_t mod ) const;
+        Unit * FindUID( const uint32_t uid ) const;
 
         void SortFastest();
     };
@@ -119,6 +119,7 @@ namespace Battle
     {
     public:
         Force( Army & parent, bool opposite, TroopsUidGenerator & generator );
+
         Force( const Force & ) = delete;
 
         ~Force() override;
@@ -136,7 +137,7 @@ namespace Battle
         uint32_t GetDeadHitPoints() const;
         uint32_t GetDeadCounts() const;
 
-        int GetColor() const;
+        PlayerColor GetColor() const;
         int GetControl() const;
 
         // Returns the cost of surrender (in units of gold) for the current army on the battlefield
@@ -155,5 +156,3 @@ namespace Battle
         std::vector<uint32_t> uids;
     };
 }
-
-#endif
