@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2024                                             *
+ *   Copyright (C) 2019 - 2025                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2010 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -35,9 +35,9 @@
 #include "audio.h"
 #include "audio_manager.h"
 #include "battle.h"
-#include "battle_arena.h"
+#include "battle_arena.h" // IWYU pragma: associated
 #include "battle_army.h"
-#include "battle_interface.h"
+#include "battle_interface.h" // IWYU pragma: associated
 #include "color.h"
 #include "cursor.h"
 #include "dialog.h"
@@ -46,7 +46,6 @@
 #include "game.h"
 #include "game_delays.h"
 #include "game_hotkeys.h"
-#include "gamedefs.h"
 #include "heroes.h"
 #include "heroes_base.h"
 #include "icn.h"
@@ -67,6 +66,7 @@
 #include "translations.h"
 #include "ui_button.h"
 #include "ui_dialog.h"
+#include "ui_language.h"
 #include "ui_option_item.h"
 #include "ui_text.h"
 #include "ui_tool.h"
@@ -76,7 +76,6 @@ namespace
 {
     // DialogBattleSummary text related values
     const int bsTextWidth = 303;
-    const int bsTextXOffset = 25;
     const int bsTextYOffset = 160;
     const int bsTextIndent = 30;
 
@@ -90,7 +89,7 @@ namespace
             , _finished( false )
             , _loop( loop )
         {
-            _frameId = ICN::AnimationFrame( _icnId, 1, _counter );
+            _frameId = ICN::getAnimatedIcnIndex( _icnId, 1, _counter );
         }
 
         uint32_t frameId()
@@ -99,11 +98,11 @@ namespace
                 return _frameId;
 
             ++_counter;
-            uint32_t nextId = ICN::AnimationFrame( _icnId, 1, _counter );
+            uint32_t nextId = ICN::getAnimatedIcnIndex( _icnId, 1, _counter );
             if ( nextId < _frameId ) {
                 if ( _loop ) {
                     _counter = 0;
-                    nextId = ICN::AnimationFrame( _icnId, 1, _counter );
+                    nextId = ICN::getAnimatedIcnIndex( _icnId, 1, _counter );
                     std::swap( nextId, _frameId );
                     return nextId;
                 }
@@ -178,6 +177,31 @@ namespace
         std::queue<LoopedAnimation> _queue;
     };
 
+    void setupAnimation( fheroes2::Display & display, const fheroes2::Rect & animationRoi, LoopedAnimationSequence & sequence )
+    {
+        sequence.push( ICN::WINCMBT, true ); // needs specific for battle summary
+        const fheroes2::Sprite & sequenceBase = fheroes2::AGG::GetICN( sequence.id(), 0 );
+        const fheroes2::Sprite & sequenceStart = fheroes2::AGG::GetICN( sequence.id(), 1 );
+        Copy( sequenceBase, 0, 0, display, animationRoi.x, animationRoi.y, sequenceBase.width(), sequenceBase.height() );
+        fheroes2::Blit( sequenceStart, display, animationRoi.x + sequenceStart.x(), animationRoi.y + sequenceStart.y() );
+    }
+
+    void updateAnimation( fheroes2::Display & display, int & lastSequence, LoopedAnimationSequence & sequence, const fheroes2::Rect & animationRoi )
+    {
+        if ( Game::validateAnimationDelay( Game::BATTLE_DIALOG_DELAY ) && !sequence.nextFrame() ) {
+            if ( lastSequence != sequence.id() ) {
+                lastSequence = sequence.id();
+                const fheroes2::Sprite & base = fheroes2::AGG::GetICN( lastSequence, 0 );
+
+                Copy( base, 0, 0, display, animationRoi.x + base.x(), animationRoi.y + base.y(), base.width(), base.height() );
+            }
+            const fheroes2::Sprite & sequenceCurrent = fheroes2::AGG::GetICN( sequence.id(), sequence.frameId() );
+
+            fheroes2::Blit( sequenceCurrent, display, animationRoi.x + sequenceCurrent.x(), animationRoi.y + sequenceCurrent.y() );
+            display.render( animationRoi );
+        }
+    }
+
     enum class DialogAction : int
     {
         Open,
@@ -245,33 +269,29 @@ namespace
     DialogAction openBattleOptionDialog( bool & saveConfiguration )
     {
         fheroes2::Display & display = fheroes2::Display::instance();
-        LocalEvent & le = LocalEvent::Get();
-        Settings & conf = Settings::Get();
 
         // Set the cursor image. This dialog is called from the battlefield and does not require a cursor restorer.
         // Battlefield event processor will set the appropriate cursor after this dialog is closed.
         Cursor::Get().SetThemes( Cursor::POINTER );
 
+        fheroes2::StandardWindow background( 289, 382, true, display );
+
+        const fheroes2::Rect windowRoi = background.activeArea();
+
+        Settings & conf = Settings::Get();
         const bool isEvilInterface = conf.isEvilInterfaceEnabled();
 
-        const fheroes2::Sprite & dialog = fheroes2::AGG::GetICN( ( isEvilInterface ? ICN::SPANBKGE : ICN::SPANBKG ), 0 );
-        const fheroes2::Sprite & dialogShadow = fheroes2::AGG::GetICN( ( isEvilInterface ? ICN::SPANBKGE : ICN::SPANBKG ), 1 );
+        fheroes2::Button buttonOk;
+        const int buttonOkIcnId = isEvilInterface ? ICN::BUTTON_SMALL_OKAY_EVIL : ICN::BUTTON_SMALL_OKAY_GOOD;
+        background.renderButton( buttonOk, buttonOkIcnId, 0, 1, { 0, 5 }, fheroes2::StandardWindow::Padding::BOTTOM_CENTER );
 
-        const fheroes2::Point dialogOffset( ( display.width() - dialog.width() ) / 2, ( display.height() - dialog.height() ) / 2 );
-        const fheroes2::Point shadowOffset( dialogOffset.x - BORDERWIDTH, dialogOffset.y );
-
-        fheroes2::ImageRestorer back( display, shadowOffset.x, shadowOffset.y, dialog.width() + BORDERWIDTH, dialog.height() + BORDERWIDTH );
-        const fheroes2::Rect pos_rt( dialogOffset.x, dialogOffset.y, dialog.width(), dialog.height() );
-
-        fheroes2::Fill( display, pos_rt.x, pos_rt.y, pos_rt.width, pos_rt.height, 0 );
-        fheroes2::Blit( dialogShadow, display, pos_rt.x - BORDERWIDTH, pos_rt.y + BORDERWIDTH );
-        fheroes2::Blit( dialog, display, pos_rt.x, pos_rt.y );
+        fheroes2::ImageRestorer emptyDialogRestorer( display, windowRoi.x, windowRoi.y, windowRoi.width, windowRoi.height );
 
         const fheroes2::Sprite & panelSprite = fheroes2::AGG::GetICN( ICN::CSPANEL, 0 );
         const int32_t panelWidth = panelSprite.width();
         const int32_t panelHeight = panelSprite.height();
 
-        const fheroes2::Point optionOffset( 36 + pos_rt.x, 47 + pos_rt.y );
+        const fheroes2::Point optionOffset( windowRoi.x + 20, windowRoi.y + 31 );
         const fheroes2::Point optionStep( 92, 110 );
 
         std::vector<fheroes2::Rect> optionAreas;
@@ -283,16 +303,13 @@ namespace
             }
         }
 
-        const fheroes2::Point buttonOffset( 112 + pos_rt.x, 362 + pos_rt.y );
-        fheroes2::Button buttonOkay( buttonOffset.x, buttonOffset.y, isEvilInterface ? ICN::BUTTON_SMALL_OKAY_EVIL : ICN::BUTTON_SMALL_OKAY_GOOD, 0, 1 );
-        buttonOkay.draw();
-
         RedrawBattleSettings( optionAreas );
 
-        display.render();
+        display.render( background.totalArea() );
 
+        LocalEvent & le = LocalEvent::Get();
         while ( le.HandleEvents() ) {
-            le.isMouseLeftButtonPressedInArea( buttonOkay.area() ) ? buttonOkay.drawOnPress() : buttonOkay.drawOnRelease();
+            buttonOk.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( buttonOk.area() ) );
 
             bool redrawScreen = false;
 
@@ -334,15 +351,16 @@ namespace
             else if ( le.MouseClickLeft( optionAreas[6] ) ) {
                 return DialogAction::AudioSettings;
             }
+
             if ( le.MouseClickLeft( optionAreas[7] ) ) {
                 return DialogAction::HotKeys;
             }
+
             if ( le.MouseClickLeft( optionAreas[8] ) ) {
                 conf.setBattleDamageInfo( !conf.isBattleShowDamageInfoEnabled() );
                 redrawScreen = true;
             }
-
-            if ( le.isMouseRightButtonPressedInArea( optionAreas[0] ) ) {
+            else if ( le.isMouseRightButtonPressedInArea( optionAreas[0] ) ) {
                 fheroes2::showStandardTextMessage( _( "Speed" ), _( "Set the speed of combat actions and animations." ), 0 );
             }
             else if ( le.isMouseRightButtonPressedInArea( optionAreas[1] ) ) {
@@ -375,19 +393,18 @@ namespace
             else if ( le.isMouseRightButtonPressedInArea( optionAreas[8] ) ) {
                 fheroes2::showStandardTextMessage( _( "Damage Info" ), _( "Toggle to display damage information during the battle." ), 0 );
             }
-            else if ( le.isMouseRightButtonPressedInArea( buttonOkay.area() ) ) {
+            else if ( le.isMouseRightButtonPressedInArea( buttonOk.area() ) ) {
                 fheroes2::showStandardTextMessage( _( "Okay" ), _( "Exit this menu." ), 0 );
             }
 
-            if ( Game::HotKeyCloseWindow() || le.MouseClickLeft( buttonOkay.area() ) ) {
+            if ( Game::HotKeyCloseWindow() || le.MouseClickLeft( buttonOk.area() ) ) {
                 break;
             }
 
             if ( redrawScreen ) {
-                fheroes2::Blit( dialog, display, pos_rt.x, pos_rt.y );
+                emptyDialogRestorer.restore();
                 RedrawBattleSettings( optionAreas );
-                buttonOkay.draw();
-                display.render();
+                display.render( emptyDialogRestorer.rect() );
 
                 saveConfiguration = true;
             }
@@ -552,11 +569,7 @@ bool Battle::Arena::DialogBattleSummary( const Result & res, const std::vector<A
         sequence.push( ICN::UNKNOWN, false );
     }
 
-    // Setup animation
-    const fheroes2::Sprite & sequenceBase = fheroes2::AGG::GetICN( sequence.id(), 0 );
-    const fheroes2::Sprite & sequenceStart = fheroes2::AGG::GetICN( sequence.id(), 1 );
-    Copy( sequenceBase, 0, 0, display, animationRoi.x, animationRoi.y, sequenceBase.width(), sequenceBase.height() );
-    fheroes2::Blit( sequenceStart, display, animationRoi.x + sequenceStart.x(), animationRoi.y + sequenceStart.y() );
+    setupAnimation( display, animationRoi, sequence );
 
     const fheroes2::Rect summaryRoi( roi.x + 11, roi.y + bsTextYOffset, roi.width - 22, roi.height - bsTextYOffset );
     fheroes2::ImageRestorer summaryBackground( display, summaryRoi.x, summaryRoi.y, roi.width, summaryRoi.height );
@@ -647,21 +660,28 @@ bool Battle::Arena::DialogBattleSummary( const Result & res, const std::vector<A
         = allowToRestart ? fheroes2::StandardWindow::Padding::BOTTOM_LEFT : fheroes2::StandardWindow::Padding::BOTTOM_CENTER;
     background.renderButton( buttonOk, buttonOkICN, 0, 1, { buttonHorizontalMargin, buttonVerticalMargin }, buttonOkPadding );
 
-    display.render( background.totalArea() );
+    if ( Game::validateDisplayFadeIn() ) {
+        fheroes2::fadeInDisplay();
+    }
+    else {
+        display.render( background.totalArea() );
+    }
 
     LocalEvent & le = LocalEvent::Get();
 
     int sequenceId = sequence.id();
 
     while ( le.HandleEvents() ) {
-        le.isMouseLeftButtonPressedInArea( buttonOk.area() ) ? buttonOk.drawOnPress() : buttonOk.drawOnRelease();
+        buttonOk.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( buttonOk.area() ) );
+
         if ( allowToRestart ) {
-            le.isMouseLeftButtonPressedInArea( buttonRestart->area() ) ? buttonRestart->drawOnPress() : buttonRestart->drawOnRelease();
+            buttonRestart->drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( buttonRestart->area() ) );
         }
 
         if ( Game::HotKeyCloseWindow() || le.MouseClickLeft( buttonOk.area() ) ) {
             break;
         }
+
         if ( le.isMouseRightButtonPressedInArea( buttonOk.area() ) ) {
             fheroes2::showStandardTextMessage( _( "Okay" ), _( "Click to leave the battle results." ), Dialog::ZERO );
         }
@@ -675,19 +695,12 @@ bool Battle::Arena::DialogBattleSummary( const Result & res, const std::vector<A
             }
         }
 
-        // Animation
-        if ( Game::validateAnimationDelay( Game::BATTLE_DIALOG_DELAY ) && !sequence.nextFrame() ) {
-            if ( sequenceId != sequence.id() ) {
-                sequenceId = sequence.id();
-                const fheroes2::Sprite & base = fheroes2::AGG::GetICN( sequenceId, 0 );
+        updateAnimation( display, sequenceId, sequence, animationRoi );
+    }
 
-                Copy( base, 0, 0, display, animationRoi.x + base.x(), animationRoi.y + base.y(), base.width(), base.height() );
-            }
-            const fheroes2::Sprite & sequenceCurrent = fheroes2::AGG::GetICN( sequence.id(), sequence.frameId() );
-
-            fheroes2::Blit( sequenceCurrent, display, animationRoi.x + sequenceCurrent.x(), animationRoi.y + sequenceCurrent.y() );
-            display.render( animationRoi );
-        }
+    // Free memory because RESTART button is not used more.
+    if ( allowToRestart ) {
+        buttonRestart.reset();
     }
 
     if ( !artifacts.empty() ) {
@@ -783,7 +796,7 @@ bool Battle::Arena::DialogBattleSummary( const Result & res, const std::vector<A
             }
 
             while ( le.HandleEvents() ) {
-                le.isMouseLeftButtonPressedInArea( buttonOk.area() ) ? buttonOk.drawOnPress() : buttonOk.drawOnRelease();
+                buttonOk.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( buttonOk.area() ) );
 
                 // Display captured artifact info on right click
                 if ( le.isMouseRightButtonPressedInArea( artifactArea ) ) {
@@ -792,22 +805,12 @@ bool Battle::Arena::DialogBattleSummary( const Result & res, const std::vector<A
                 else if ( le.MouseClickLeft( buttonOk.area() ) || Game::HotKeyCloseWindow() ) {
                     break;
                 }
-                else if ( le.isMouseRightButtonPressedInArea( buttonOk.area() ) ) {
+
+                if ( le.isMouseRightButtonPressedInArea( buttonOk.area() ) ) {
                     fheroes2::showStandardTextMessage( _( "Okay" ), _( "Exit this menu." ), Dialog::ZERO );
                 }
-                // Animation
-                if ( Game::validateAnimationDelay( Game::BATTLE_DIALOG_DELAY ) && !sequence.nextFrame() ) {
-                    if ( sequenceId != sequence.id() ) {
-                        sequenceId = sequence.id();
-                        const fheroes2::Sprite & base = fheroes2::AGG::GetICN( sequenceId, 0 );
 
-                        Copy( base, 0, 0, display, animationRoi.x + base.x(), animationRoi.y + base.y(), base.width(), base.height() );
-                    }
-                    const fheroes2::Sprite & sequenceCurrent = fheroes2::AGG::GetICN( sequence.id(), sequence.frameId() );
-
-                    fheroes2::Blit( sequenceCurrent, display, animationRoi.x + sequenceCurrent.x(), animationRoi.y + sequenceCurrent.y() );
-                    display.render( animationRoi );
-                }
+                updateAnimation( display, sequenceId, sequence, animationRoi );
             }
         }
     }
@@ -819,42 +822,26 @@ void Battle::Arena::DialogBattleNecromancy( const uint32_t raiseCount )
     // Set the cursor image. This dialog does not require a cursor restorer.
     Cursor::Get().SetThemes( Cursor::POINTER );
 
-    const bool isEvilInterface = Settings::Get().isEvilInterfaceEnabled();
-    const fheroes2::Sprite & dialog = fheroes2::AGG::GetICN( ( isEvilInterface ? ICN::WINLOSEE : ICN::WINLOSE ), 0 );
-    const fheroes2::Sprite & dialogShadow = fheroes2::AGG::GetICN( ( isEvilInterface ? ICN::WINLOSEE : ICN::WINLOSE ), 1 );
-
     fheroes2::Display & display = fheroes2::Display::instance();
-    const fheroes2::Point dialogOffset( ( display.width() - dialog.width() ) / 2, ( display.height() - dialog.height() ) / 2 );
-    const fheroes2::Point shadowOffset( dialogOffset.x - BORDERWIDTH, dialogOffset.y );
+    fheroes2::StandardWindow background( 287, 424, true, display );
 
-    fheroes2::ImageRestorer back( display, shadowOffset.x, shadowOffset.y, dialog.width() + BORDERWIDTH, dialog.height() + BORDERWIDTH - 1 );
-    const fheroes2::Rect renderArea( dialogOffset.x, dialogOffset.y, dialog.width(), dialog.height() );
+    // Animation border
+    const bool isEvilInterface = Settings::Get().isEvilInterfaceEnabled();
+    const fheroes2::Sprite & originalBorderImage = fheroes2::AGG::GetICN( isEvilInterface ? ICN::WINLOSEE : ICN::WINLOSE, 0 );
+    const fheroes2::Rect animationBorderRoi{ 43, 32, 231, 133 };
 
-    fheroes2::Blit( dialogShadow, display, renderArea.x - BORDERWIDTH, renderArea.y + BORDERWIDTH - 1 );
-    fheroes2::Blit( dialog, display, renderArea.x, renderArea.y );
+    const fheroes2::Rect & roi( background.activeArea() );
+    const fheroes2::Rect animationRoi( roi.x + ( ( roi.width - animationBorderRoi.width ) / 2 ) + 4, roi.y + 20, animationBorderRoi.width, animationBorderRoi.height );
+    Copy( originalBorderImage, animationBorderRoi.x, animationBorderRoi.y, display, animationRoi.x - 4, animationRoi.y - 4, animationRoi.width, animationRoi.height );
 
     LoopedAnimationSequence sequence;
-    sequence.push( ICN::WINCMBT, true );
+    setupAnimation( display, animationRoi, sequence );
 
-    if ( sequence.isFinished() ) // Cannot be!
-        sequence.push( ICN::UNKNOWN, false );
-
-    const fheroes2::Sprite & sequenceBase = fheroes2::AGG::GetICN( sequence.id(), 0 );
-    const fheroes2::Sprite & sequenceStart = fheroes2::AGG::GetICN( sequence.id(), 1 );
-
-    const fheroes2::Point sequenceRenderAreaOffset( 47, 36 );
-
-    fheroes2::Blit( sequenceBase, display, renderArea.x + sequenceRenderAreaOffset.x + sequenceBase.x(), renderArea.y + sequenceRenderAreaOffset.y + sequenceBase.y() );
-    fheroes2::Blit( sequenceStart, display, renderArea.x + sequenceRenderAreaOffset.x + sequenceStart.x(),
-                    renderArea.y + sequenceRenderAreaOffset.y + sequenceStart.y() );
-
-    int xOffset = renderArea.x + bsTextXOffset;
-    int yOffset = renderArea.y + bsTextYOffset + 15;
-
-    const int adjustedTextWidth = bsTextWidth - 33;
+    // Text stuff
+    int yOffset = animationRoi.y + animationRoi.height + 8;
 
     const fheroes2::Text titleBox( _( "Necromancy!" ), fheroes2::FontType::normalYellow() );
-    titleBox.draw( xOffset, yOffset + 2, adjustedTextWidth, display );
+    titleBox.draw( roi.x, yOffset, roi.width, display );
 
     const Monster mons( Monster::SKELETON );
     std::string msg = _( "Practicing the dark arts of necromancy, you are able to raise %{count} of the enemy's dead to return under your service as %{monster}." );
@@ -863,55 +850,51 @@ void Battle::Arena::DialogBattleNecromancy( const uint32_t raiseCount )
 
     const fheroes2::Text messageBox( msg, fheroes2::FontType::normalWhite() );
     yOffset += bsTextIndent;
-    messageBox.draw( xOffset, yOffset + 2, adjustedTextWidth, display );
+    const int32_t messageWidth = roi.width - 22;
+    messageBox.draw( roi.x + 11, yOffset, messageWidth, display );
 
     const fheroes2::Sprite & monsterSprite = fheroes2::AGG::GetICN( ICN::MONS32, mons.GetSpriteIndex() );
-    yOffset += messageBox.height( adjustedTextWidth ) + monsterSprite.height();
+    yOffset += messageBox.height( messageWidth ) + monsterSprite.height() - 2;
     fheroes2::Blit( monsterSprite, display, ( display.width() - monsterSprite.width() ) / 2, yOffset );
 
     fheroes2::Text raiseCountText( std::to_string( raiseCount ), fheroes2::FontType::smallWhite() );
-    raiseCountText.fitToOneRow( adjustedTextWidth );
+    raiseCountText.fitToOneRow( roi.width );
     yOffset += 30;
     raiseCountText.draw( ( display.width() - raiseCountText.width() ) / 2, yOffset + 2, display );
+
     Game::PlayPickupSound();
 
-    const int buttonOffset = 121;
-    const int buttonICN = isEvilInterface ? ICN::BUTTON_SMALLER_OKAY_EVIL : ICN::BUTTON_SMALLER_OKAY_GOOD;
-    fheroes2::Button buttonOk( renderArea.x + buttonOffset, renderArea.y + 410, buttonICN, 0, 1 );
-    buttonOk.draw();
+    // Button
+    const int buttonICN = isEvilInterface ? ICN::BUTTON_SMALL_OKAY_EVIL : ICN::BUTTON_SMALL_OKAY_GOOD;
+    fheroes2::Button buttonOk;
+    background.renderButton( buttonOk, buttonICN, 0, 1, { 0, 5 }, fheroes2::StandardWindow::Padding::BOTTOM_CENTER );
 
-    display.render();
+    display.render( roi );
 
     LocalEvent & le = LocalEvent::Get();
+
+    int sequenceId = sequence.id();
+
     while ( le.HandleEvents() ) {
-        le.isMouseLeftButtonPressedInArea( buttonOk.area() ) ? buttonOk.drawOnPress() : buttonOk.drawOnRelease();
+        buttonOk.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( buttonOk.area() ) );
 
-        // exit
-        if ( Game::HotKeyCloseWindow() || le.MouseClickLeft( buttonOk.area() ) )
+        if ( Game::HotKeyCloseWindow() || le.MouseClickLeft( buttonOk.area() ) ) {
             break;
-
-        // animation
-        if ( Game::validateAnimationDelay( Game::BATTLE_DIALOG_DELAY ) && !sequence.nextFrame() ) {
-            const fheroes2::Sprite & base = fheroes2::AGG::GetICN( sequence.id(), 0 );
-            const fheroes2::Sprite & sequenceCurrent = fheroes2::AGG::GetICN( sequence.id(), sequence.frameId() );
-
-            fheroes2::Blit( base, display, renderArea.x + sequenceRenderAreaOffset.x + sequenceBase.x(), renderArea.y + sequenceRenderAreaOffset.y + sequenceBase.y() );
-            fheroes2::Blit( sequenceCurrent, display, renderArea.x + sequenceRenderAreaOffset.x + sequenceCurrent.x(),
-                            renderArea.y + sequenceRenderAreaOffset.y + sequenceCurrent.y() );
-            display.render();
         }
+
+        updateAnimation( display, sequenceId, sequence, animationRoi );
     }
 }
 
-int Battle::Arena::DialogBattleHero( const HeroBase & hero, const bool buttons, Status & status ) const
+int Battle::Arena::DialogBattleHero( HeroBase & hero, const bool buttons, Status & status ) const
 {
     const Settings & conf = Settings::Get();
 
     Cursor & cursor = Cursor::Get();
     cursor.SetThemes( Cursor::POINTER );
 
-    const int currentColor = GetCurrentColor();
-    const bool readonly = currentColor != hero.GetColor() || !buttons;
+    const PlayerColor currentColor = GetCurrentColor();
+    const bool readonly = ( currentColor != hero.GetColor() || !buttons );
     const fheroes2::Sprite & dialog = fheroes2::AGG::GetICN( ( conf.isEvilInterfaceEnabled() ? ICN::VGENBKGE : ICN::VGENBKG ), 0 );
 
     const fheroes2::Point dialogShadow( 15, 15 );
@@ -929,10 +912,9 @@ int Battle::Arena::DialogBattleHero( const HeroBase & hero, const bool buttons,
     pos_rt.width -= dialogShadow.x;
 
     const fheroes2::Rect portraitArea( pos_rt.x + 7, pos_rt.y + 35, 113, 108 );
-    const Heroes * actionHero = ( currentColor == hero.GetColor() ) ? dynamic_cast<const Heroes *>( &hero ) : nullptr;
 
     hero.PortraitRedraw( pos_rt.x + 12, pos_rt.y + 42, PORT_BIG, display );
-    int col = ( Color::NONE == hero.GetColor() ? 1 : Color::GetIndex( hero.GetColor() ) + 1 );
+    const int col = ( PlayerColor::NONE == hero.GetColor() ? 1 : Color::GetIndex( hero.GetColor() ) + 1 );
     fheroes2::Blit( fheroes2::AGG::GetICN( ICN::VIEWGEN, col ), display, pos_rt.x + 133, pos_rt.y + 36 );
 
     std::string str = hero.isCaptain() ? _( "Captain of %{name}" ) : _( "%{name} the %{race}" );
@@ -992,66 +974,89 @@ int Battle::Arena::DialogBattleHero( const HeroBase & hero, const bool buttons,
     fheroes2::Button btnSurrender( pos_rt.x + 133, pos_rt.y + 148, ICN::VIEWGEN, 13, 14 );
     fheroes2::Button btnClose( pos_rt.x + 192, pos_rt.y + 148, ICN::VIEWGEN, 15, 16 );
 
-    if ( readonly || !hero.HaveSpellBook() || hero.Modes( Heroes::SPELLCASTED ) )
+    if ( readonly || !hero.HaveSpellBook() || hero.Modes( Heroes::SPELLCASTED ) ) {
         btnCast.disable();
+    }
 
-    if ( readonly || !CanRetreatOpponent( hero.GetColor() ) )
+    if ( readonly || !CanRetreatOpponent( hero.GetColor() ) ) {
         btnRetreat.disable();
+    }
 
-    if ( readonly || !CanSurrenderOpponent( hero.GetColor() ) )
+    if ( readonly || !CanSurrenderOpponent( hero.GetColor() ) ) {
         btnSurrender.disable();
+    }
 
     btnCast.draw();
     btnRetreat.draw();
     btnSurrender.draw();
-    btnClose.draw();
 
     int result = 0;
 
-    display.render( pos_rt );
-
-    std::string statusMessage = _( "Hero's Options" );
-
     LocalEvent & le = LocalEvent::Get();
-    while ( le.HandleEvents() && !result ) {
-        btnCast.isEnabled() && le.isMouseLeftButtonPressedInArea( btnCast.area() ) ? btnCast.drawOnPress() : btnCast.drawOnRelease();
-        btnRetreat.isEnabled() && le.isMouseLeftButtonPressedInArea( btnRetreat.area() ) ? btnRetreat.drawOnPress() : btnRetreat.drawOnRelease();
-        btnSurrender.isEnabled() && le.isMouseLeftButtonPressedInArea( btnSurrender.area() ) ? btnSurrender.drawOnPress() : btnSurrender.drawOnRelease();
-        le.isMouseLeftButtonPressedInArea( btnClose.area() ) ? btnClose.drawOnPress() : btnClose.drawOnRelease();
 
-        if ( buttons ) {
-            // The Cast Spell is available for a hero and a captain.
-            if ( le.isMouseCursorPosInArea( btnCast.area() ) && currentColor == hero.GetColor() ) {
-                statusMessage = _( "Cast Spell" );
-            }
-            // The retreat is available during a player's turn only. A captain cannot retreat.
-            else if ( le.isMouseCursorPosInArea( btnRetreat.area() ) && currentColor == hero.GetColor() && !hero.isCaptain() ) {
-                statusMessage = _( "Retreat" );
-            }
-            // The surrender is available during a player's turn only. A captain cannot surrender.
-            else if ( le.isMouseCursorPosInArea( btnSurrender.area() ) && currentColor == hero.GetColor() && !hero.isCaptain() ) {
-                statusMessage = _( "Surrender" );
-            }
-            else if ( le.isMouseCursorPosInArea( btnClose.area() ) ) {
-                statusMessage = _( "Cancel" );
-            }
-            // The Hero Screen is available for a Hero only (not Captain) and when UI is not read-only.
-            else if ( le.isMouseCursorPosInArea( portraitArea ) && actionHero != nullptr && actionHero->isHeroes() && !readonly ) {
-                statusMessage = _( "Hero Screen" );
-            }
-            else if ( hero.isCaptain() ) {
-                statusMessage = _( "Captain's Options" );
-            }
-            else {
-                statusMessage = _( "Hero's Options" );
-            }
-        }
-        else {
+    if ( !buttons ) {
+        // This is a case when this dialog was called by the right mouse button press.
+
+        btnClose.disable();
+        btnClose.draw();
+
+        display.render( pos_rt );
+
+        while ( le.HandleEvents() ) {
             if ( !le.isMouseRightButtonPressed() ) {
                 break;
             }
+        }
 
-            continue;
+        return result;
+    }
+
+    btnClose.draw();
+
+    display.render( pos_rt );
+
+    // The Hero Screen is available for a Hero only (not Captain) and only when the corresponding player has a turn.
+    Heroes * heroForHeroScreen = ( currentColor == hero.GetColor() ) ? dynamic_cast<Heroes *>( &hero ) : nullptr;
+
+    std::string statusMessage;
+
+    while ( le.HandleEvents() && !result ) {
+        if ( btnCast.isEnabled() ) {
+            btnCast.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( btnCast.area() ) );
+        }
+        if ( btnRetreat.isEnabled() ) {
+            btnRetreat.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( btnRetreat.area() ) );
+        }
+        if ( btnSurrender.isEnabled() ) {
+            btnSurrender.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( btnSurrender.area() ) );
+        }
+
+        btnClose.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( btnClose.area() ) );
+
+        // The Cast Spell is available for a hero and a captain.
+        if ( le.isMouseCursorPosInArea( btnCast.area() ) && currentColor == hero.GetColor() ) {
+            statusMessage = _( "Cast Spell" );
+        }
+        // The retreat is available during a player's turn only. A captain cannot retreat.
+        else if ( le.isMouseCursorPosInArea( btnRetreat.area() ) && currentColor == hero.GetColor() && !hero.isCaptain() ) {
+            statusMessage = _( "Retreat" );
+        }
+        // The surrender is available during a player's turn only. A captain cannot surrender.
+        else if ( le.isMouseCursorPosInArea( btnSurrender.area() ) && currentColor == hero.GetColor() && !hero.isCaptain() ) {
+            statusMessage = _( "Surrender" );
+        }
+        else if ( le.isMouseCursorPosInArea( btnClose.area() ) ) {
+            statusMessage = _( "Cancel" );
+        }
+        // The Hero Screen is available for a Hero only (not Captain) and only when the corresponding player has a turn.
+        else if ( le.isMouseCursorPosInArea( portraitArea ) && heroForHeroScreen != nullptr ) {
+            statusMessage = _( "Hero Screen" );
+        }
+        else if ( hero.isCaptain() ) {
+            statusMessage = _( "Captain's Options" );
+        }
+        else {
+            statusMessage = _( "Hero's Options" );
         }
 
         if ( Game::HotKeyCloseWindow() || le.MouseClickLeft( btnClose.area() ) ) {
@@ -1070,10 +1075,10 @@ int Battle::Arena::DialogBattleHero( const HeroBase & hero, const bool buttons,
             result = 3;
         }
 
-        if ( le.MouseClickLeft( portraitArea ) && actionHero != nullptr ) {
+        if ( le.MouseClickLeft( portraitArea ) && heroForHeroScreen != nullptr ) {
             LocalEvent::Get().reset();
-            // IMPORTANT!!! This is extremely dangerous but we have no choice with current code. Make sure that this trick doesn't allow user to modify the hero.
-            const_cast<Heroes *>( actionHero )->OpenDialog( true, true, true, true, false, false );
+
+            heroForHeroScreen->OpenDialog( true, true, true, true, false, false, fheroes2::getLanguageFromAbbreviation( conf.getGameLanguage() ) );
 
             // Fade-in to restore the screen after closing the hero dialog.
             fheroes2::fadeInDisplay( _interface->GetInterfaceRoi(), !display.isDefaultSize() );
@@ -1096,16 +1101,16 @@ int Battle::Arena::DialogBattleHero( const HeroBase & hero, const bool buttons,
                 _( "Surrendering costs gold. However if you pay the ransom, the hero and all of his or her surviving creatures will be available to recruit again. The cost of surrender is half of the total cost of the non-temporary troops remaining in the army." ),
                 Dialog::ZERO );
         }
-        else if ( le.isMouseRightButtonPressedInArea( portraitArea ) && actionHero != nullptr ) {
+        else if ( le.isMouseRightButtonPressedInArea( portraitArea ) && heroForHeroScreen != nullptr ) {
             fheroes2::showStandardTextMessage( _( "Hero Screen" ), _( "Open Hero Screen to view full information about the hero." ), Dialog::ZERO );
         }
         else if ( le.isMouseRightButtonPressedInArea( btnClose.area() ) ) {
             fheroes2::showStandardTextMessage( _( "Cancel" ), _( "Return to the battle." ), Dialog::ZERO );
         }
 
-        if ( statusMessage != status.GetMessage() ) {
-            status.SetMessage( statusMessage );
-            status.Redraw( display );
+        if ( statusMessage != status.getMessage() ) {
+            status.setMessage( std::move( statusMessage ), false );
+            status.redraw( display );
             display.render( status );
         }
     }
@@ -1115,8 +1120,12 @@ int Battle::Arena::DialogBattleHero( const HeroBase & hero, const bool buttons,
 
 bool Battle::DialogBattleSurrender( const HeroBase & hero, uint32_t cost, Kingdom & kingdom )
 {
-    if ( kingdom.GetColor() == hero.GetColor() ) // this is weird. You're surrending to yourself!
+    if ( kingdom.GetColor() == hero.GetColor() ) {
+        // This is weird. You're surrendering to yourself!
+        assert( 0 );
+
         return false;
+    }
 
     fheroes2::Display & display = fheroes2::Display::instance();
     LocalEvent & le = LocalEvent::Get();
@@ -1165,16 +1174,15 @@ bool Battle::DialogBattleSurrender( const HeroBase & hero, uint32_t cost, Kingdo
     btnAccept.draw();
     btnDecline.draw();
 
-    const auto drawGoldMsg = [cost, &kingdom, &btnAccept]() {
+    const auto drawGoldMsg = [cost, &kingdom, &display]( const fheroes2::Rect & btnAcceptArea ) {
         std::string str = _( "Not enough gold (%{gold})" );
 
         StringReplace( str, "%{gold}", cost - kingdom.GetFunds().gold );
 
         const fheroes2::Text text( str, fheroes2::FontType::smallWhite() );
-        const fheroes2::Rect rect = btnAccept.area();
 
         // Since button area includes 3D effect on the left side we need to shift the text by X axis to center it in relation to the button.
-        text.draw( rect.x + ( rect.width - text.width() ) / 2 + 2, rect.y - 13, fheroes2::Display::instance() );
+        text.draw( btnAcceptArea.x + ( btnAcceptArea.width - text.width() ) / 2 + 2, btnAcceptArea.y - 13, display );
     };
 
     const int icn = isEvilInterface ? ICN::SURRENDE : ICN::SURRENDR;
@@ -1197,7 +1205,7 @@ bool Battle::DialogBattleSurrender( const HeroBase & hero, uint32_t cost, Kingdo
     fheroes2::ImageRestorer back( display, pos_rt.x, pos_rt.y, pos_rt.width, pos_rt.height );
 
     if ( !kingdom.AllowPayment( Funds( Resource::GOLD, cost ) ) ) {
-        drawGoldMsg();
+        drawGoldMsg( btnAccept.area() );
     }
 
     display.render();
@@ -1205,12 +1213,15 @@ bool Battle::DialogBattleSurrender( const HeroBase & hero, uint32_t cost, Kingdo
     bool result = false;
 
     while ( le.HandleEvents() && !result ) {
-        if ( btnAccept.isEnabled() )
-            le.isMouseLeftButtonPressedInArea( btnAccept.area() ) ? btnAccept.drawOnPress() : btnAccept.drawOnRelease();
-        le.isMouseLeftButtonPressedInArea( btnDecline.area() ) ? btnDecline.drawOnPress() : btnDecline.drawOnRelease();
+        if ( btnAccept.isEnabled() ) {
+            btnAccept.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( btnAccept.area() ) );
+        }
 
-        if ( btnMarket.isEnabled() )
-            le.isMouseLeftButtonPressedInArea( btnMarket.area() ) ? btnMarket.drawOnPress() : btnMarket.drawOnRelease();
+        btnDecline.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( btnDecline.area() ) );
+
+        if ( btnMarket.isEnabled() ) {
+            btnMarket.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( btnMarket.area() ) );
+        }
 
         if ( btnAccept.isEnabled() && ( Game::HotKeyPressEvent( Game::HotKeyEvent::DEFAULT_OKAY ) || le.MouseClickLeft( btnAccept.area() ) ) ) {
             result = true;
@@ -1227,7 +1238,7 @@ bool Battle::DialogBattleSurrender( const HeroBase & hero, uint32_t cost, Kingdo
             else {
                 btnAccept.disable();
 
-                drawGoldMsg();
+                drawGoldMsg( btnAccept.area() );
             }
 
             btnAccept.draw();
