@@ -44,7 +44,7 @@ enum class PlayerColor : uint8_t;
 
 namespace Rand
 {
-    class DeterministicRandomGenerator;
+    class PCG32;
 }
 
 namespace Battle
@@ -123,8 +123,8 @@ namespace Battle
             _mirrorUnit = ptr;
         }
 
-        void SetRandomMorale( Rand::DeterministicRandomGenerator & randomGenerator );
-        void SetRandomLuck( Rand::DeterministicRandomGenerator & randomGenerator );
+        void SetRandomMorale( Rand::PCG32 & randomGenerator );
+        void SetRandomLuck( Rand::PCG32 & randomGenerator );
         void NewTurn();
 
         bool isFlying() const;
@@ -192,7 +192,7 @@ namespace Battle
         // is set to true, then the value of 'skipMovedCheck' doesn't matter.
         uint32_t GetSpeed( const bool skipStandingCheck, const bool skipMovedCheck ) const;
 
-        uint32_t GetDamage( const Unit & enemy, Rand::DeterministicRandomGenerator & randomGenerator ) const;
+        uint32_t GetDamage( const Unit & enemy, Rand::PCG32 & randomGenerator ) const;
 
         // Returns the threat level of this unit, calculated as if it attacked the 'defender' unit. See
         // the implementation for details.
@@ -306,7 +306,7 @@ namespace Battle
         void PostKilledAction();
 
         uint32_t GetMagicResist( const Spell & spell, const HeroBase * applyingHero ) const;
-        Spell GetSpellMagic( Rand::DeterministicRandomGenerator & randomGenerator ) const;
+        Spell GetSpellMagic( Rand::PCG32 & randomGenerator ) const;
 
         const HeroBase * GetCommander() const;
         // If the color of the current unit is valid (i.e. this unit is not under the influence of a Berserker spell), then returns the commander of the army with the
