@@ -255,15 +255,13 @@ bool Dialog::inputString( const fheroes2::TextBase & title, const fheroes2::Text
     dst_pt.y = frameBoxArea.y + frameBoxArea.height - cancelButtonIcn.height();
     fheroes2::Button buttonCancel( dst_pt.x, dst_pt.y, cancelButtonIcnID, 0, 1 );
 
-    // Generate a button to open the Virtual Keyboard window.
-    fheroes2::Sprite releasedVirtualKB;
-    fheroes2::Sprite pressedVirtualKB;
-    const fheroes2::Size buttonVirtualKBSize{ 40, 25 };
+    const int buttonVirtualKBIcnID = isEvilInterface ? ICN::BUTTON_VIRTUAL_KEYBOARD_EVIL : ICN::BUTTON_VIRTUAL_KEYBOARD_GOOD;
+    const fheroes2::Sprite & buttonVirtualKBIcn = fheroes2::AGG::GetICN( buttonVirtualKBIcnID, 0 );
 
-    makeButtonSprites( releasedVirtualKB, pressedVirtualKB, "...", buttonVirtualKBSize, isEvilInterface, isEvilInterface ? ICN::UNIFORMBAK_EVIL : ICN::UNIFORMBAK_GOOD );
-    // To center the button horizontally we have to take into account that actual button sprite is 10 pixels longer then the requested button width.
-    fheroes2::ButtonSprite buttonVirtualKB = makeButtonWithBackground( frameBoxArea.x + ( frameBoxArea.width - buttonVirtualKBSize.width - 10 ) / 2, dst_pt.y - 30,
-                                                                       releasedVirtualKB, pressedVirtualKB, display );
+    dst_pt.x = frameBoxArea.x + ( frameBoxArea.width - buttonVirtualKBIcn.width() ) / 2;
+    dst_pt.y -= 30;
+    // This dialog uses the "uniform" background so the pressed button sprite ID is 2.
+    fheroes2::Button buttonVirtualKB( dst_pt.x, dst_pt.y, buttonVirtualKBIcnID, 0, 2 );
 
     if ( result.empty() ) {
         buttonOk.disable();
