@@ -26,7 +26,6 @@
 #include <algorithm>
 #include <array>
 #include <cmath>
-#include <cstddef>
 #include <iterator>
 #include <map>
 #include <set>
@@ -1958,16 +1957,6 @@ void Heroes::LevelUpSecondarySkill( const HeroSeedsForLevelUp & seeds, int prima
     }
 }
 
-void Heroes::ApplyPenaltyMovement( uint32_t penalty )
-{
-    if ( move_point >= penalty ) {
-        move_point -= penalty;
-    }
-    else {
-        move_point = 0;
-    }
-}
-
 bool Heroes::MayStillMove( const bool ignorePath, const bool ignoreSleeper ) const
 {
     if ( !isActive() ) {
